/*
Wind monitor for WM version2 series

Created by Steve Nishiarai 2/Feb/2022
*/

#define VERSION "wm10m220210"

/*
LCD display has 16 columns X 2 lines

Display example of the display state

 0123456789ABCDEF
 ----------------
|NNW10.2  25.0 |  Inatantaneous wind direction and speed, Temperature
|WNW 9.5   W11.0 |  Average wind direction and speed, Maximum wind direction and speed
 ----------------

Display example of the averaging time setup state
 0123456789ABCDEF
 ----------------
|Averaging time  |
|10 minutes      |
 ----------------

Display example of the Instantaneous wind speed alarm setup state
 0123456789ABCDEF
 ----------------
|Wind speed alarm|
|Inst 10.0 m/s   |
 ----------------

Display example of the maximum wind speed alarm setup state 
 0123456789ABCDEF
 ----------------
|Wind speed alarm|
|Max  10.0 m/s   |
 ----------------
*/

//---------------------------------------------
//for using mySerial
#include <stdio.h>
//for using eeprom
#include <EEPROM.h>

//---------------------------------------------
#include <LiquidCrystal.h>
#include <SoftwareSerial.h>
#include <DFR_Key.h>

//---------------------------------------------
//EEPROM map

//0 Buffer of averaging count
byte AvrMinute ;
//1 Buffer of instantaneous alarm wind speed
byte InstWindSpeedAlarm ;
//2 Buffer of maximum alarm wind speed
byte MaxWindSpeedAlarm ;
//3 display state
byte State ;
//4 Buffer of the graph scale
byte GraphScale ;

//Initial averaging time
#define INITAVRMINUTE 10 // 10minutes
//Initial alarm wind speed
#define INITALARMSPEED 10 // 10m/s

//---------------------------------------------
//Display state definitions
#define STATE_MESR_AVR    0  //Measurement and show average wind
#define STATE_MESR_GRAPH  1  //Measurement and show graph
#define STATE_SET_AVERAGE  2  //Set averaging minute
#define STATE_SET_INSTWIN  3  //Set instantaneous wind speed alarm point
#define STATE_SET_MAXWIN  4  //Set maximum wind speed alarm point
#define STATE_SET_SCALE    5  //Set graph scale
#define STATE_END          6  //End of state

//---------------------------------------------

SoftwareSerial mySerial(2, 3); // RX, TX

//Pin assignments for wind speed alarm
#define InvalidAlarmPin  11
#define MaxAlarmPin      12
#define InstAlarmPin    13

//Pin assignments for SainSmart LCD Keypad Shield
LiquidCrystal lcd(8, 9, 4, 5, 6, 7); 
// make a cute degree symbol
uint8_t degree[8]  = {140,146,146,140,128,128,128,128};
// make a graph block symbol
uint8_t block[8]  = {127,127,127,127,127,127,127,127};

DFR_Key keypad;

int localKey = 0;
String keyString = "";

//Assuming analog read should be more than this value when no key input
#define NOKEYTHRESH 900

//define constant
#define CONST_2PID360 0.01745329251 // (2*pai)/360
#define CONST_360D2PI 57.29577951 // 360/(2*pai)
//Buffer of the absolute and XY vector average value of windspeed
float AWindSpeed = 0;
float XWindSpeed = 0;
float YWindSpeed = 0;
//Averaging counter
unsigned int AvrCounter = 0;
//Buffer of the average wind speed and direction
float AvrWindSpeed = 0;
float AvrWindAngle = 0;
//Buffer of the maximum windspeed and direction
float MaxWindSpeed = 0;
float MaxWindAngle = 0;
//Graphic peak wind speed buffer
unsigned int PeakWindSpeed = 0 ;
//Flag of indicate average data aquired
boolean FlagAverage = false ;


/***********************************************
  split string with delimiter
    int <number of devided array> = split(<string>, <delimiter>, <array of devided string>, <maximum array size>) ;
*/
int split(String data, char delimiter, String *dst, int arraySize)
{
  // clear the array
  for (int i = 0; i < arraySize; i++)
  {
    dst[i] = "\0" ;
  }
  int index = 0;
  int datalength = data.length();
  for (int i = 0; i < datalength; i++)
  {
    char tmp = data.charAt(i);
    if ( tmp == delimiter )
    {
      index++;
      if ( index > (arraySize - 1)) return -1;
    }
    else dst[index] += tmp;
  }
  return (index + 1);
}


//Show Invalid message at the place of instantaneous wind direction and speed
void ShowInvalid()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Invalid");
}


void ShowWind (float WindAngle, float WindSpeed)
{
  //Convert degree angle to 16 cardinal direction
  String str ;
  if ( WindAngle > 11.25 *  1 && WindAngle <= 11.25 *  3 ) { str = "NNE" ;}
  if ( WindAngle > 11.25 *  3 && WindAngle <= 11.25 *  5 ) { str = " NE" ;}
  if ( WindAngle > 11.25 *  5 && WindAngle <= 11.25 *  7 ) { str = "ENE" ;}
  if ( WindAngle > 11.25 *  7 && WindAngle <= 11.25 *  9 ) { str = "  E" ;}
  if ( WindAngle > 11.25 *  9 && WindAngle <= 11.25 * 11 ) { str = "ESE" ;}
  if ( WindAngle > 11.25 * 11 && WindAngle <= 11.25 * 13 ) { str = " SE" ;}
  if ( WindAngle > 11.25 * 13 && WindAngle <= 11.25 * 15 ) { str = "SSE" ;}
  if ( WindAngle > 11.25 * 15 && WindAngle <= 11.25 * 17 ) { str = "  S" ;}
  if ( WindAngle > 11.25 * 17 && WindAngle <= 11.25 * 19 ) { str = "SSW" ;}
  if ( WindAngle > 11.25 * 19 && WindAngle <= 11.25 * 21 ) { str = " SW" ;}
  if ( WindAngle > 11.25 * 21 && WindAngle <= 11.25 * 23 ) { str = "WSW" ;}
  if ( WindAngle > 11.25 * 23 && WindAngle <= 11.25 * 25 ) { str = "  W" ;}
  if ( WindAngle > 11.25 * 25 && WindAngle <= 11.25 * 27 ) { str = "WNW" ;}
  if ( WindAngle > 11.25 * 27 && WindAngle <= 11.25 * 29 ) { str = " NW" ;}
  if ( WindAngle > 11.25 * 29 && WindAngle <= 11.25 * 31 ) { str = "NNW" ;}
  if ( WindAngle > 11.25 * 31 || WindAngle <= 11.25 *  1 ) { str = "  N" ;}

  lcd.print(str);
  //Convert floating value to integer
  int IWindSpeed = WindSpeed * 10 ;
  //separate to upper value and lower value with decimal point
  int UWindSpeed = IWindSpeed / 10 ;
  int LWindSpeed = IWindSpeed - UWindSpeed * 10 ;
  if ( UWindSpeed < 10 )
  {
    lcd.print(" ");
  }
  lcd.print(UWindSpeed);
  lcd.print(".");
  lcd.print(LWindSpeed);
  lcd.print(" ");
}


void ShowAveragingTime()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Averaging period");
  lcd.setCursor(0, 1);
  if ( AvrMinute < 10 ) { lcd.print(" ") ;}
  lcd.print(AvrMinute);
  lcd.print(" minutes");
}


void ShowAlarmInstSpeed()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Wind speed alarm");
  lcd.setCursor(0, 1);
  lcd.print("Inst ");
  if ( InstWindSpeedAlarm < 10 ) { lcd.print(" ") ;}
  lcd.print(InstWindSpeedAlarm);
  lcd.print(" m/s");
}


void ShowAlarmMaxSpeed()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Wind speed alarm");
  lcd.setCursor(0, 1);
  lcd.print("Max  ");
  if ( MaxWindSpeedAlarm < 10 ) { lcd.print(" ") ;}
  lcd.print(MaxWindSpeedAlarm);
  lcd.print(" m/s");
}


void ShowScale()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Graph scale");
  lcd.setCursor(0, 1);
  lcd.print("1 dot = 0.");
  lcd.print(GraphScale);
  lcd.print(" m/s");
}


void A0test()
{
//  lcd.print("A0 value");
  int valueMid = 0 ;
  int valueThresh = 0 ;

  while(1)
  {
    int valueMax = 0 ;
    int valueMin = 1024 ;
//    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Push key ");
    lcd.setCursor(0, 1);
    lcd.print("    ");
    lcd.setCursor(0, 1);
    lcd.print( valueMid );
    lcd.setCursor(13, 1);
    lcd.print( valueThresh );
    while ( analogRead(A0) <= NOKEYTHRESH ){}
    while ( analogRead(A0) > NOKEYTHRESH ){}
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Keep push   ");
    int i = 0 ;
    while (1)
    {
      int value = analogRead(A0) ;
      if ( value > NOKEYTHRESH ) { break ;}
      if ( value > valueMax ) { valueMax = value ;}
      if ( value < valueMin ) { valueMin = value ;}
      lcd.setCursor(0, 1);
      lcd.print( value );
      lcd.print("   ");
      lcd.setCursor(4, 1);
      lcd.print( valueMax );
      lcd.print("   ");
      lcd.setCursor(8, 1);
      lcd.print( valueMin );
      lcd.print("   ");
      lcd.setCursor(10, 0);
      if ( i < 10 ) { lcd.print(" ") ;}
      lcd.print( i );
      i++ ;
      delay(200);
    }
    valueThresh = ( valueMax - valueMin ) ;
    valueMid = valueMin + valueThresh / 2 ;
  }
}


void setup() 
{
  pinMode(InstAlarmPin, OUTPUT); 
  pinMode(MaxAlarmPin, OUTPUT); 
  pinMode(InvalidAlarmPin, OUTPUT); 

  // start serial port:
  Serial.begin(9600);
  mySerial.begin(9600);
  
  lcd.begin(16, 2);
  // create a cute degree symbol
  lcd.createChar(0, degree);
  // create a graph block symbol
  lcd.createChar(1, block);

  lcd.clear();

  if ( analogRead(A0) < NOKEYTHRESH )
  {
    A0test();
  }

  lcd.setCursor(0, 0);
  lcd.print(VERSION);
  //Show for 2 second
  delay(2000);
  lcd.clear();

  //read averaging munites from eeprom
  AvrMinute = EEPROM.read(0);
  //more than 99 munites is not allowed
  if ( AvrMinute > 99 )
  {
    //set to default
    AvrMinute = INITAVRMINUTE ;
    EEPROM.write(0, AvrMinute);
  }
  ShowAveragingTime();
  //Show for 2 second
  delay(2000);
  lcd.clear();

  //read instantaneous wind speed alarm value from eeprom
  InstWindSpeedAlarm = EEPROM.read(1);
  //more than 99 m/s is not allowed
  if ( InstWindSpeedAlarm > 99 )
  {
    //set to default
    InstWindSpeedAlarm = INITALARMSPEED ;
    EEPROM.write(1, InstWindSpeedAlarm);
  }
  ShowAlarmInstSpeed();
  //Show for 2 second
  delay(2000);
  lcd.clear();

  //read maximum wind speed alarm value from eeprom
  MaxWindSpeedAlarm = EEPROM.read(2);
  //more than 99 m/s is not allowed
  if ( MaxWindSpeedAlarm > 99 )
  {
    //set to default
    MaxWindSpeedAlarm = INITALARMSPEED ;
    EEPROM.write(2, MaxWindSpeedAlarm);
  }
  ShowAlarmMaxSpeed();
  //Show for 2 second
  delay(2000);
  lcd.clear();

  //read start state from eeprom
  State = EEPROM.read(3);
  //Start state should be 0 or 1
  if ( State > 1 )
  {
    //set to default
    State = STATE_MESR_AVR ;
    EEPROM.write(3, State);
  }

  //read graph scale from eeprom
  GraphScale = EEPROM.read(4);
  //GraphScale should be less than 10
  if ( GraphScale > 10 )
  {
    //set to default
    GraphScale = 1 ;
    EEPROM.write(4, GraphScale);
  }
  ShowScale();
  //Show for 2 second
  delay(2000);
  lcd.clear();
}


void loop() 
{
  if ( State == STATE_MESR_AVR || State == STATE_MESR_GRAPH )
  {
/****************************************************************************************************
 * WM10 Transmission format(NMEA0183 compliant)
 *
 *  'c' represents single ascii character, 'd' represents decimal number and 'h' represents one digit of hexadecimal code in capital letter.
 *------------------------------------------------------------------------------
 *        1 2 3   4 5 6
 *        | | |   | | |
 * $WIMWV,d,c,d.d,c,c*hh<CR><LF>
 *------------------------------------------------------------------------------
 *
 *1. Wind Angle, 0 to 359 degrees
 *2. Reference, R = Relative, T = True (fixed to R)
 *3. Wind Speed
 *4. Wind Speed Units, K/M/N (fixed to M)
 *5. Status, A = Data Valid, V = Data Invalid
 *6. Checksum

 *------------------------------------------------------------------------------
 *        1   2 3
 *        |   | |
 * $WIMTA,d.d,c*hh<CR><LF>
 *------------------------------------------------------------------------------
 *
 *1. Temperature
 *2. Units, C = Celsius (fixed to C)
 *3. Checksum
*/
    //check serial
    if (mySerial.available())
    {
      //read 1 line from WM10
      String str = mySerial.readStringUntil('\n');
      //discard CR code
      str.trim();
      //Pass the line to PC
      Serial.println(str);

      String items[6] ; // array for store the divided strings
      split(str, ',', items, 6) ;
      if ( items[0].equals("$WIMWV") )
      {
        String SubItems[2] ;
        split(items[5], '*', SubItems, 2) ;
        //If the status is "V"(invalid) then initialize averaging buffer.
        if ( SubItems[0].equals("V") )
        {
          digitalWrite(InvalidAlarmPin, HIGH);
          AvrCounter = 0;
          AWindSpeed = 0;
          XWindSpeed = 0;
          YWindSpeed = 0;
          AvrWindSpeed = 0;
          MaxWindSpeed = 0;
          FlagAverage = false ;
          //Show Invalid
          ShowInvalid() ;
        }
        else
        {
          digitalWrite(InvalidAlarmPin, LOW);
          float WindAngle ;
          float WindSpeed ;
          //convert wind speed and direction to floating value
          WindAngle = items[1].toFloat() ;
          WindSpeed = items[3].toFloat();
          //Show instantaneous wind direction and speed
          lcd.setCursor(0, 0);
          ShowWind (WindAngle, WindSpeed);
          if ( WindSpeed >= InstWindSpeedAlarm )
          {
            digitalWrite(InstAlarmPin, HIGH);
          }
          else
          {
            digitalWrite(InstAlarmPin, LOW);
          }

          if ( State == STATE_MESR_GRAPH )
          {
            //Calculate graph position
            unsigned int GraphWindSpeed = WindSpeed * 10 / GraphScale ;
            unsigned int GraphBlockNum = GraphWindSpeed / 5 ;
            unsigned int GraphBlockPos = GraphWindSpeed - GraphBlockNum * 5 ;
            //Limit block number to 16
            if ( GraphBlockNum > 16 )
            {
              lcd.setCursor(0, 1);
              for ( unsigned int i=0; i<16; i++ )
              {
                lcd.write((byte)1);
              }
              //clear the peak wind buffer
              PeakWindSpeed = 0 ;
            }
            else
            {
              //Calculate peak position
              if ( PeakWindSpeed < GraphWindSpeed ) { PeakWindSpeed = GraphWindSpeed ;}
              unsigned int PeakBlockNum = PeakWindSpeed / 5 ;
              unsigned int PeakBlockPos = PeakWindSpeed - PeakBlockNum * 5 ;
              //Generate end of graph image
              byte graph[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
              byte Pattern = B100000 ;
  
              for ( unsigned int i=0; i<GraphBlockPos; i++ )
              {
                for ( unsigned int j=0; j<8; j++ )
                {
                  graph[j] |= Pattern ;
                  graph[j] = graph[j] >> 1 ;
                }
              }
  
              //Generate peak image
              byte peak[8] ;
  
              PeakBlockPos++ ; //0...4 -> 1...5
              Pattern = Pattern >> PeakBlockPos ;
              for ( unsigned int j=0; j<8; j++ )
              {
                peak[j] = Pattern ;
              }
              //Create peak symbol
              lcd.createChar(3, peak);
  
              //if the peak symbol place is same to the end of graph
              if ( PeakBlockNum == GraphBlockNum )
              {
                //Merge symbols
                for ( unsigned int j=0; j<8; j++ )
                {
                  graph[j] = graph[j] | peak[j]  ;
                }
              }
              //Create end of graph symbol
              lcd.createChar(2, graph);
  
              //Show graph block
              lcd.setCursor(0, 1);
              for ( unsigned int i=0; i<GraphBlockNum; i++ )
              {
                lcd.write((byte)1);
              }
              //Show end of graph
              lcd.write((byte)2);
  
              if ( PeakBlockNum > GraphBlockNum )
              {
                //Fill left space to peak symbol
                for ( unsigned int i=GraphBlockNum+1; i<PeakBlockNum; i++ )
                {
                  lcd.print(" ");
                }
                //Show peak symbol
                lcd.write((byte)3);
                //Fill right space form peak symbol
                for ( unsigned int i=PeakBlockNum+1; i<16; i++ )
                {
                  lcd.print(" ");
                }
              }
              else
              {
                //Fill right space from end of graph
                for ( unsigned int i=GraphBlockNum+1; i<16; i++ )
                {
                  lcd.print(" ");
                }
              }
            }
          }

          //Keep maximum wind speed and direction
          if ( WindSpeed > MaxWindSpeed )
          {
            MaxWindSpeed = WindSpeed ;
            MaxWindAngle = WindAngle ;
          }
          //Accumulate wind speed
          float WindAngleRad = WindAngle * CONST_2PID360 ;
          AWindSpeed += WindSpeed ;
          XWindSpeed += WindSpeed * cos(WindAngleRad) ;
          YWindSpeed += WindSpeed * sin(WindAngleRad) ;

          //Convert averaging minutes to count (mulitiple value of 3 seconds, 60sec / 3sec = 20)
          unsigned int AvrCount = AvrMinute * 20 ;
          //if averaging time of data has been received then
          if ( ++AvrCounter >= AvrCount )
          {
            FlagAverage = true ;
            //Calculate average wind speed and direction
            AvrWindSpeed = AWindSpeed / AvrCount ;
            XWindSpeed /= AvrCount ;
            YWindSpeed /= AvrCount ;
            AvrWindAngle = atan2(YWindSpeed,XWindSpeed) * CONST_360D2PI ;
            if ( AvrWindAngle < 0 ) { AvrWindAngle += 360 ;}

            if ( State == STATE_MESR_AVR )
            {
              //Show average wind direction and speed
              lcd.setCursor(0, 1);
              ShowWind (AvrWindAngle, AvrWindSpeed);
              //Show maximum wind direction and speed
              lcd.setCursor(8, 1);
              ShowWind (MaxWindAngle, MaxWindSpeed);
            }

            if ( MaxWindSpeed >= MaxWindSpeedAlarm )
            {
              digitalWrite(MaxAlarmPin, HIGH);
            }
            else
            {
              digitalWrite(MaxAlarmPin, LOW);
            }

            //Initialize averaging buffer
            AvrCounter = 0;
            AWindSpeed = 0;
            XWindSpeed = 0;
            YWindSpeed = 0;
            MaxWindSpeed = 0;
          }
        }
      }
      else if ( items[0].equals("$WIMTA") )
      {
        //copy temperature to buffer
        float Temperature = items[1].toFloat();
        //Show temperature
        lcd.setCursor(8, 0);
        //Convert floating value to integer
        int ITemperature = Temperature * 10 ;
        //separate to upper value and lower value with decimal point
        int UTemperature = ITemperature / 10 ;
        int LTemperature = ITemperature - UTemperature * 10 ;
        if ( LTemperature < 0 )
        {
          LTemperature *= -1 ;
        }
        if ( UTemperature >= 0 )
        {
          lcd.print(" ");
        }
        if ( UTemperature < 10 || UTemperature > -10 )
        {
          lcd.print(" ");
        }
        lcd.print(UTemperature);
        lcd.print(".");
        lcd.print(LTemperature);
        lcd.print(" ");
        lcd.write((byte)0);
        lcd.print("C ");
      }
    }
  }
  /*
keypad.getKey();
Grabs the current key.
Return Key Codes (in left-to-right order):

Busy   - -1 SAMPLE_WAIT
None   - 0 NO_KEY
Select - 1 SELECT_KEY
Left   - 2 LEFT_KEY
Up     - 3 UP_KEY
Down   - 4 DOWN_KEY
Right  - 5 RIGHT_KEY
  */
  localKey = keypad.getKey();
  if ( localKey == SELECT_KEY )
  {
    lcd.clear();
    State++ ;
    if ( State == STATE_END )
    {
      State = 0 ;
      if ( FlagAverage == true )
      {
        //Show average wind direction and speed
        lcd.setCursor(0, 1);
        ShowWind (AvrWindAngle, AvrWindSpeed);
        //Show maximum wind direction and speed
        lcd.setCursor(8, 1);
        ShowWind (MaxWindAngle, MaxWindSpeed);
      }
    }
    if ( State == STATE_SET_AVERAGE )
    {
      ShowAveragingTime();
    }
    if ( State == STATE_SET_INSTWIN )
    {
      ShowAlarmInstSpeed();
    }
    if ( State == STATE_SET_MAXWIN )
    {
      ShowAlarmMaxSpeed();
    }
    if ( State == STATE_SET_SCALE )
    {
      ShowScale();
    }

  }
  if ( localKey == UP_KEY )
  {
    if ( State == STATE_SET_AVERAGE )
    {
      if ( ++AvrMinute > 99 )
      {
        AvrMinute = 1 ;
      }
      ShowAveragingTime();
    }
    if ( State == STATE_SET_INSTWIN )
    {
      if ( ++InstWindSpeedAlarm > 99 )
      {
        AvrMinute = 1 ;
      }
      ShowAlarmInstSpeed();
    }
    if ( State == STATE_SET_MAXWIN )
    {
      if ( ++MaxWindSpeedAlarm > 99 )
      {
        MaxWindSpeedAlarm = 1 ;
      }
      ShowAlarmMaxSpeed();
    }
    if ( State == STATE_SET_SCALE )
    {
      if ( ++GraphScale > 9 )
      {
        GraphScale = 1 ;
      }
      ShowScale();
    }
  }
  if ( localKey == DOWN_KEY )
  {
    if ( State == STATE_SET_AVERAGE )
    {
      if ( --AvrMinute == 0 )
      {
        AvrMinute = 99 ;
      }
      ShowAveragingTime();
    }
    if ( State == STATE_SET_INSTWIN )
    {
      if ( --InstWindSpeedAlarm == 0 )
      {
        AvrMinute = 99 ;
      }
      ShowAlarmInstSpeed();
    }
    if ( State == STATE_SET_MAXWIN )
    {
      if ( --MaxWindSpeedAlarm == 0 )
      {
        MaxWindSpeedAlarm = 99 ;
      }
      ShowAlarmMaxSpeed();
    }
    if ( State == STATE_SET_SCALE )
    {
      if ( --GraphScale == 0 )
      {
        GraphScale = 9 ;
      }
      ShowScale();
    }
  }
  if ( localKey == RIGHT_KEY )
  {
    if ( State == STATE_MESR_AVR )
    {
      lcd.setCursor(0, 0);
      lcd.print("Set to default  ");
      delay(1000);
      if ( STATE_MESR_AVR != EEPROM.read(3) )
      {
        EEPROM.write(3, STATE_MESR_AVR);
      }
    }
    if ( State == STATE_MESR_GRAPH )
    {
      lcd.setCursor(0, 0);
      lcd.print("Set to default  ");
      delay(1000);
      if ( STATE_MESR_GRAPH != EEPROM.read(3) )
      {
        EEPROM.write(3, STATE_MESR_GRAPH);
      }
    }
    if ( State == STATE_SET_AVERAGE )
    {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Write setting");
      delay(1000);
      //Write setting to eeprom if modified
      if ( AvrMinute != EEPROM.read(0) )
      {
        EEPROM.write(0, AvrMinute);
      }
      ShowAveragingTime();
      AvrCounter = 0;
      AWindSpeed = 0;
      XWindSpeed = 0;
      YWindSpeed = 0;
      AvrWindSpeed = 0;
      MaxWindSpeed = 0;
      FlagAverage = false ;
    }
    if ( State == STATE_SET_INSTWIN )
    {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Write setting");
      delay(1000);
      //Write setting to eeprom if modified
      if ( InstWindSpeedAlarm != EEPROM.read(1) )
      {
        EEPROM.write(1, InstWindSpeedAlarm);
       }
      ShowAlarmInstSpeed();
      digitalWrite(InstAlarmPin, LOW);
    }
    if ( State == STATE_SET_MAXWIN )
    {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Write setting");
      delay(1000);
      //Write setting to eeprom if modified
      if ( MaxWindSpeedAlarm != EEPROM.read(2) )
      {
        EEPROM.write(2, MaxWindSpeedAlarm);
      }
      ShowAlarmMaxSpeed();
      digitalWrite(MaxAlarmPin, LOW);
    }
    if ( State == STATE_SET_SCALE )
    {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Write setting");
      delay(1000);
      //Write setting to eeprom if modified
      if ( GraphScale != EEPROM.read(4) )
      {
        EEPROM.write(4, GraphScale);
      }
      ShowScale();
    }
  }
  //delay for key chattering escape
  delay (100) ;
}
